/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.util;

import java.io.UnsupportedEncodingException;
import java.text.DecimalFormatSymbols;
import java.text.Normalizer;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.mentawai.i18n.LocaleManager;

public class StringUtils {
	
	private static final String[] CHARS = split("abcdefghijklmnopqrstuvxwyz");
	
	private static final String[] NUMBERS = split("0123456789");
	
    private static final Map<Character, String> urlDirt    = new HashMap<Character, String>();
	private static final Map<Character, String> alphabetics    = new HashMap<Character, String>();
	private static final Map<Character, String> specials    = new HashMap<Character, String>();
	
  static{
    	String empty =  "";
        specials.put('-',"_");
        specials.put(' ',"-");
        specials.put('@',"_");
        specials.put('º', empty);
        specials.put('!', empty);
        specials.put('?',  empty);
        specials.put('.', empty);
        specials.put(',', empty);
        specials.put(':', empty);
        specials.put(';', empty);
        specials.put('(', empty);
        specials.put(')', empty);
        specials.put('\'', empty);
        specials.put('"', empty);
        specials.put('\\', empty);
        specials.put('/', "-");
        specials.put('<', empty);
        specials.put('>', empty);
        specials.put('\t',empty);
        specials.put('\n',empty);
        specials.put('\r',empty);
        specials.put('%',empty);
        specials.put('Ø',empty);
        specials.put('ø',empty);
        specials.put('Ð',empty);
        specials.put('ð',empty);
        specials.put('Æ',empty);
        specials.put('æ',empty);
        
        alphabetics.put('á',"a");
        alphabetics.put('â',"a");
        alphabetics.put('à',"a");
        alphabetics.put('å',"a");
        alphabetics.put('ã',"a");
        alphabetics.put('ä',"a");
        alphabetics.put('é',"e");
        alphabetics.put('ê',"e");
        alphabetics.put('è',"e");
        alphabetics.put('&',"e");
        alphabetics.put('ë',"e");
        alphabetics.put('í',"i");
        alphabetics.put('î',"i");
        alphabetics.put('ì',"i");
        alphabetics.put('ï',"i");
        alphabetics.put('ó',"o");
        alphabetics.put('ô',"o");
        alphabetics.put('ò',"o");
        alphabetics.put('õ',"o");
        alphabetics.put('ö',"o");
        alphabetics.put('ú',"u");
        alphabetics.put('û',"u");
        alphabetics.put('ù',"u");
        alphabetics.put('ü',"u");
        alphabetics.put('Ç',"c");
        alphabetics.put('ç',"c");
        alphabetics.put('ñ',"n");
		
		urlDirt.putAll(specials);
		urlDirt.putAll(alphabetics);
		
    }
	
	private static final Random RAND = new Random();
    
	
  public static final String escapeHTML(String s){
  	   StringBuffer sb = new StringBuffer();
  	   int n = s.length();
  	   for (int i = 0; i < n; i++) {
  	      char c = s.charAt(i);
  	      switch (c) {
  	         case '<': sb.append("&lt;"); break;
  	         case '>': sb.append("&gt;"); break;
  	         case '&': sb.append("&amp;"); break;
  	         case '"': sb.append("&quot;"); break;
  	         case 'à': sb.append("&agrave;");break;
  	         case 'À': sb.append("&Agrave;");break;
  	         case 'â': sb.append("&acirc;");break;
  	         case 'Â': sb.append("&Acirc;");break;
  	         case 'ä': sb.append("&auml;");break;
  	         case 'Ä': sb.append("&Auml;");break;
  	         case 'å': sb.append("&aring;");break;
  	         case 'Å': sb.append("&Aring;");break;
  	         case 'æ': sb.append("&aelig;");break;
  	         case 'Æ': sb.append("&AElig;");break;
  	         case 'ç': sb.append("&ccedil;");break;
  	         case 'Ç': sb.append("&Ccedil;");break;
  	         case 'é': sb.append("&eacute;");break;
  	         case 'É': sb.append("&Eacute;");break;
  	         case 'è': sb.append("&egrave;");break;
  	         case 'È': sb.append("&Egrave;");break;
  	         case 'ê': sb.append("&ecirc;");break;
  	         case 'Ê': sb.append("&Ecirc;");break;
  	         case 'ë': sb.append("&euml;");break;
  	         case 'Ë': sb.append("&Euml;");break;
  	         case 'ï': sb.append("&iuml;");break;
  	         case 'Ï': sb.append("&Iuml;");break;
  	         case 'ô': sb.append("&ocirc;");break;
  	         case 'Ô': sb.append("&Ocirc;");break;
  	         case 'ö': sb.append("&ouml;");break;
  	         case 'Ö': sb.append("&Ouml;");break;
  	         case 'ø': sb.append("&oslash;");break;
  	         case 'Ø': sb.append("&Oslash;");break;
  	         case 'ß': sb.append("&szlig;");break;
  	         case 'ù': sb.append("&ugrave;");break;
  	         case 'Ù': sb.append("&Ugrave;");break;         
  	         case 'û': sb.append("&ucirc;");break;         
  	         case 'Û': sb.append("&Ucirc;");break;
  	         case 'ü': sb.append("&uuml;");break;
  	         case 'Ü': sb.append("&Uuml;");break;
  	         case '®': sb.append("&reg;");break;         
  	         case '©': sb.append("&copy;");break;   
  	         case '€': sb.append("&euro;"); break;
  	         // be carefull with this one (non-breaking whitee space)
  	         case ' ': sb.append("&nbsp;");break;         
  	         
  	         default:  sb.append(c); break;
  	      }
  	   }
  	   return sb.toString();
  	}
	
    public static boolean isNotEmpty(String value) {
        return value != null && value.length() > 0;
    }
    
    /**
     * Verifica se todas as string passadas não são NULL ou Vazias. Se pelo menos uma for
     * retorna false.
     * @param values
     * @return boolean
     */
    public static boolean isNotEmpty(String ...values) {
    	for (int i = 0; i < values.length; i++) {
			String value = values[i];
			if(isEmpty(value)){
				return false;
			}
		}
    	
        return true;
    }
    
    public static boolean isEmpty(String s) {
        return s == null || s.trim().length() == 0;
    }
    
    /**
     * Turns a line with strings separated by one
     * or more spaces into a String array.
     * 
     * @param line The line to be sliced
     * @return A String array with the slices
     */
    public static String[] slice(String line) {
    	return line.trim().split("\\s+");
    }
    
    public static String[] split(String line) {
    	String[] s = new String[line.length()];
    	
    	for(int i=0;i<line.length();i++) {
    		s[i] = String.valueOf(line.charAt(i));
    	}
    	return s;
    }
    
    public static String randomString(int size) {
    	
    	return randomString(size, false);
    }
    
    public static String randomString(int size, boolean onlyChars) {
    	
    	return randomString(size, onlyChars, false);
    }
    
    public static String randomNumber(int size) {
    	
    	int count = 0;
    	
    	StringBuilder sb = new StringBuilder(size);
    	
    	while(count < size) {
    		
    		int index = RAND.nextInt(NUMBERS.length);
    		
    		sb.append(NUMBERS[index]);
    		
    		count++;
    	}
    	
    	return sb.toString();
    }
    
    public static String randomString(int size, boolean onlyChars, boolean upcase) {
    	
    	int count = 0;
    	
    	StringBuilder sb = new StringBuilder(size);
    	
    	while(count < size) {
    		
    		int index = RAND.nextInt(CHARS.length);
    		
    		sb.append(CHARS[index]);
    		
    		count++;
    		
    		if (!onlyChars) {
    			
    			if (count >= size) break;
    		
	    		index = RAND.nextInt(NUMBERS.length);
	    		
	    		sb.append(NUMBERS[index]);
	    		
	    		count++;
    		}
    	}
    	
    	return upcase ? sb.toString().toUpperCase() : sb.toString();
    }
    
    private static byte toByte(char hex) {
    	
    	switch(hex) {
    	
	    	case '0':
	    		return 0;
	    	case '1':
	    		return 1;
	    	case '2':
	    		return 2;
	    	case '3':
	    		return 3;
	    	case '4':
	    		return 4;
	    	case '5':
	    		return 5;
	    	case '6':
	    		return 6;
	    	case '7':
	    		return 7;
	    	case '8':
	    		return 8;
	    	case '9':
	    		return 9;
	    	case 'A':
	    	case 'a':
	    		return 10;
	    	case 'B':
	    	case 'b':
	    		return 11;
	    	case 'C':
	    	case 'c':
	    		return 12;
	    	case 'D':
	    	case 'd':
	    		return 13;
	    	case 'E':
	    	case 'e':
	    		return 14;
	    	case 'F':
	    	case 'f':
	    		return 15;
	    		
			default:
				throw new IllegalArgumentException("Not a valid hex digit: " + hex);
    	}
    }
    
    
	/**
	 * Método que converte de UTF-8 para ISO-8859-1
	 * 
	 * @param text String
	 * @return String
	 */
	public static String UTF8toISO88591(String text) throws UnsupportedEncodingException {
		byte p[] = text.getBytes("UTF-8");  
		return new String(p, 0, p.length, "ISO-8859-1");
	}

	/**
	 * Método que converte de ISO-8859-1 para UTF-8
	 * 
	 * @param text String
	 * @return String
	 * @throws Exception
	 */
	public static String ISO88591toUTF8(String text) throws UnsupportedEncodingException {
		byte p[] = text.getBytes("ISO-8859-1");
		return new String(p, 0, p.length, "UTF-8");
	}	
	
    /**
     * Assinatura dos bytes de uma String UTF-8
     * 
     * EF BB BF
     */
    private static final byte UTF8Signature[] = {
        -17, -69, -65
    };
	
    public static boolean isUTF8( byte[] buf ) {
        return isUTF8( buf, 0, buf.length );
    }
    
    /**
     * Método que recebe um array de bytes e verifica se esse array
     * de bytes de uma respectiva String é UTF-8
     * 
     * @param buf byte[]
     * @param offset int
     * @param length int
     * @return boolean
     */
    public static boolean isUTF8( byte[] buf, int offset, int length ) {
        
    	// verificando se começa com a assinatura do UTF-8
    	if( startsWith( buf, offset, UTF8Signature ) ) {
            return true; // caso identifique inicialmente que é UTF-8
        }
        
    	// caso não identifique inicialmente a String como UTF-8
    	// definindo um flag para indicar se é ou não UTF-8
        boolean yesItIs = false;
        // percorre os bytes do array da String
        for(int i = offset; i < offset + length; i++) {
            
        	if( ( buf[i] & 0xc0 ) != 192 ) { continue; }
            
            int nBytes = 2;
            
            do {                
            	if(nBytes >= 8) { break; }                
                int mask = 1 << 7 - nBytes;
                if((buf[i] & mask) == 0) { break; }
                nBytes++;                
            } while( true );
            
            for( int j = 1; j < nBytes; j++ ) {
                if( i + j >= length || ( buf[i + j] & 0xc0 ) != 128 ) {
                    return false;
                }
            }

            // se passar por toda a verificação de bytes acima, então é UTF-8
            yesItIs = true;
        }

        // retorna o valor do flag
        return yesItIs;
    }
    
    
    /**
     * Método que verifica se começa com a respectiva assinatura de UTF-8
     * 
     * @param buf byte[]
     * @param offset int
     * @param prefix byte[]
     * @return boolean
     */
    private static boolean startsWith( byte[] buf, int offset, byte[] prefix ) {
        // verifica se a assinatura não corresponde
    	for( int i = 0; i < prefix.length; i++ ) {
            if( i + offset == buf.length || buf[i + offset] != prefix[i] ) {
                return false;
            }
        }
        // retorno padrão
        return true;
    }    
    
    public static String fromBytesToHex(byte[] b) {
    	
    	StringBuilder sb = new StringBuilder(b.length * 2);
    	
    	for (int i=0; i < b.length; i++) {
    		sb.append(Integer.toString( ( b[i] & 0xff ) + 0x100, 16).substring( 1 ) );
    	}
    	
    	return sb.toString();
    }
    
    public static byte[] fromHexToBytes(String hex) {
    	
    	int x = hex.length() / 2;
    	
    	byte[] b = new byte[x];
    	
    	int index = 0;
    	
    	for(int i=0;i<hex.length();i+=2) {
    		
    		byte b1 = toByte(hex.charAt(i));
    		
    		byte b2 = toByte(hex.charAt(i + 1));
    		
    		b[index++] = (byte) ((b1 << 4) + b2);
    	}
    	
    	return b;
    }
    
    public static String fromHexToString(String hex) {
    	return new String(fromHexToBytes(hex));
    }
    
    public static String fromStringToHex(String s) {
    	return fromBytesToHex(s.getBytes());
    }
    
    public static String invert(String s) {
    	StringBuilder sb = new StringBuilder(s.length());
    	
    	for(int i=s.length()-1;i>=0;i--) {
    		sb.append(s.charAt(i));
    	}
    	
    	return sb.toString();
    }
    
    /**
	 * Convert Money value as String to a Number
	 * @param value
	 * @param loc (Optional) Default is LocaleManager.DEFAULT_LOCALE
	 */
	public static Number parseCurrencyNumber(String value, Locale loc) throws ParseException {
		if(loc == null) loc = LocaleManager.DEFAULT_LOCALE;
		String symbol = DecimalFormatSymbols.getInstance(loc).getCurrencySymbol();
		if("R$".equals(symbol)) symbol += " "; // Caso seja a moeda Real.
		Number number = NumberFormat.getCurrencyInstance(loc).parse(symbol + value);
		return number;		
	}
	
	public static String replace(String strIn, Map<Character, String> dirt) {
		if(isEmpty(strIn)) return "";

		StringBuilder outBuffer = new StringBuilder();
		String clean;
		for (Character charDirty : strIn.toCharArray()) {

			if(dirt.containsKey(charDirty)){
				clean = dirt.get(charDirty);
			}else{
				clean = charDirty.toString();
			}
			outBuffer.append(clean);
		}
		return outBuffer.toString();
	}
	
	
	/**
	 * Devolve a string informada sem acentuacao, pontuacao e sem caracteres especiais
	 */
	public static String urlFormat(String strIn){
		strIn = strIn.toLowerCase().trim();
		return StringUtils.replace(strIn, urlDirt);
	}
	
    /**  
     * Remove os caracteres especiais da string. Por padrão tambem remove os espaços em branco.
     * @see #removeSpecialChars(String, boolean) */
    public static String removeSpecialChars(String s) {
    	return removeSpecialChars(s, true);
    }
    
     
    /**
     * Remove Caracteres especiais como !,@,#,$,%,^,& e substitue os Acentos.
     */
    public static String removeSpecialChars(String s, boolean removeBlank) {
    	if(removeBlank) s = s.replaceAll(" ", "");
    	s = StringUtils.replace(s, urlDirt);
    	return s;
    }
	
	/**
	 * Remove accentued from a string and replace with ascii equivalent
	 */
	public static String removeAccents(String text) {
	 text = Normalizer.normalize(text, Normalizer.Form.NFD); 
	 return text.replaceAll("[^\\p{ASCII}]", "");
	}
	
	
	  /**
     * Underscore the given word.
     * @param word The word
     * @return The underscored version of the word
     */
    public static String underscore(String word) {
        String firstPattern = "([A-Z]+)([A-Z][a-z])";
        String secondPattern = "([a-z\\d])([A-Z])";
        String replacementPattern = "$1_$2";
        word = word.replaceAll("\\.", "/"); // replace package separator with slash
        word = word.replaceAll("\\$", "__"); // replace $ with two underscores for inner classes
        word = word.replaceAll(firstPattern, replacementPattern); // replace capital letter with _ plus lowercase letter
        word = word.replaceAll(secondPattern, replacementPattern);
        word = word.replace('-', '_');
        word = word.toLowerCase();
        return word;
    }

    public static String camelize(String word) {
        return camelize(word, false);
    }

    public static String camelize(String word, boolean lowercaseFirstLetter) {
        // replace all slashes with dots (package separator)
        Pattern p = Pattern.compile("\\/(.?)");
        Matcher m = p.matcher(word);
        while (m.find()) {
            word = m.replaceFirst("." + m.group(1)/*.toUpperCase()*/);
            m = p.matcher(word);
        }

        // uppercase the class name
        p = Pattern.compile("(\\.?)(\\w)([^\\.]*)$");
//        System.out.println("Does " + word + " match " + p + "?");
        m = p.matcher(word);
        if (m.find()) {
//            System.out.println("match! group count: " + m.groupCount());
//            for (int i = 1; i <= m.groupCount(); i++) {
//                System.out.println("group " + i + "=" + m.group(i));
//            }
            String rep = m.group(1) + m.group(2).toUpperCase() + m.group(3);
//            System.out.println("replacement string raw: " + rep);
            rep = rep.replaceAll("\\$", "\\\\\\$");
//            System.out.println("replacement string processed: " + rep);
            word = m.replaceAll(rep);
        }

        // replace two underscores with $ to support inner classes
        p = Pattern.compile("(__)(.)");
        m = p.matcher(word);
        while (m.find()) {
            word = m.replaceFirst("\\$" + m.group(2).toUpperCase());
            m = p.matcher(word);
        }

        // remove all underscores
        p = Pattern.compile("(_)(.)");
        m = p.matcher(word);
        while (m.find()) {
            word = m.replaceFirst(m.group(2).toUpperCase());
            m = p.matcher(word);
        }

        if (lowercaseFirstLetter) {
            word = word.substring(0, 1).toLowerCase() + word.substring(1);
        }

        return word;

    }
    
    /**
     * Return the ordinal for the given number.
     * @param number The number
     * @return The ordinal String
     */
    public static String ordinalize(Number number) {
        return ordinalize(number.toString());
    }
    
    /**
     * Return the ordinal for the given number.
     * @param number The number
     * @return The ordinal String
     */
    public static String ordinalize(String number) {
        int[] _11through13 = {11, 12, 13};
        for (int n : _11through13) {
            int m100 = Integer.parseInt(number) % 100;
            if (Integer.toString(m100).endsWith(Integer.toString(n))) {
                return number + "th";
            }
        }

        int m = Integer.parseInt(number) % 10;
        switch (m) {
            case 1:
                return number + "st";
            case 2:
                return number + "nd";
            case 3:
                return number + "rd";
            default:
                return number + "th";
        }
    }
    
    
    public static String capitalize(String word) {
        int strLen;
        if (word == null || (strLen = word.length()) == 0) {
            return word;
        }
        return new StringBuffer(strLen)
            .append(Character.toTitleCase(word.charAt(0)))
            .append(word.substring(1))
            .toString();
    }
    
    /**
     * Convert first character to lowercase
     * Por: Ricardo
     * @since 1.0
     * @param word
     * @return decapitalized string
     */
    public static String decapitalize(String word) {
        word = word.substring(0, 1).toLowerCase() + word.substring(1);
        return word;
    }

    /**
     * Count the number of the specified characters in the String.
     * @param word The word
     * @param countThese The characters to count
     * @return The number of times the character appears
     */
    public static int count(String word, char countThese) {
        int count = 0;
        for (char c : word.toCharArray()) {
            if (c == countThese) {
                count++;
            }
        }
        return count;
    }
    
    /** 
     * @see #join(Collection, String)
     */
    public static String join(String[] c, String separator) {
    	return join(Arrays.asList(c), separator);
    }

    /**
     * Join the specified collection with the given separator.
     * @param c The collection
     * @param separator The separator
     * @return The joined String
     */
    public static String join(Collection c, String separator) {
        StringBuilder sb = new StringBuilder();
        Iterator iter = c.iterator();
        while (iter.hasNext()) {
            Object o = iter.next();
            sb.append(o.toString());
            if (iter.hasNext()) {
                sb.append(separator);
            }
        }
        return sb.toString();
    }
    
    public static void main(String[] args) {
    	
    	System.out.println(StringUtils.randomString(6, true));
    	
    	System.out.println(StringUtils.randomString(6, true, true));
    	
    	System.out.println(StringUtils.randomNumber(6));
    	
    	System.out.println(StringUtils.randomString(6));
    }
}


